/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// |
// | constants
// |

// properties
const gkoProperties       = goMnenhy.InitStringBundle("chrome://mnenhy/locale/mnenhy-headers.properties")

// container names
const gksBaseContainer    = "header"
const gksKnown            = "known";
const gksFilter           = "filter";
const gksFilterPref       = "mailnews.customHeaders";
const gksCompose          = "compose";
const gksComposePref      = "mail.compose.other.header";
const gksCustom           = "custom";
const gksFormat           = "format";
const gksDisplay          = "display";
const gksIndex            = "index";
const gksUseToggle        = "useToggle";

const gkrsHeaderVars      = ["text","mail","news","mid","link"];
const gkoHeaderPos        = {text: 0, mail: 1, news: 2, mid: 3, link: 4};
const gkrsHeader          = {text: goMnenhy.GetString("text", gkoProperties),
                             mail: goMnenhy.GetString("mail", gkoProperties),
                             news: goMnenhy.GetString("news", gkoProperties),
                             mid:  goMnenhy.GetString("mid",  gkoProperties),
                             link: goMnenhy.GetString("link", gkoProperties)  };

// styles for the listbox tree checkbox
const gksAttrProp         = "properties";
const gksAttrDisabled     = "disabled";
const gksHeaderActive     = "subscribedColumn2";
const gksHeaderActiveYes  = "subscribed";

// idefixes (wuff!)
const gksListPrefix       = "mnenhy-lh-";
const gksDropPrefix       = "mnenhy-dd-";
const gksCheckPostfix     = "-chk";
const gksFormatPostfix    = "-fmt";
const gksDisplayPostfix   = "-dsp";
const gksIndexPostfix     = "-idx";

// other stuff
const gksTreeCellImage    = "image";
const gksHeaderListSel    = "mnenhyHeaderListSelect";
const gksHeaderListDis    = "mnenhyHeaderListDisabled";
const gksViewListSel      = "mnenhyViewListSelect";


// |
// | global objects (will be set in InitMnenhyPrefHeader)
// |
var goListbox      = null;
var goDropdown     = null;
var goDropdownList = null;


// |
// | vars
// |

var gsCurrentListId = null;


// |
// | functions
// |
function InitHeaderPrefPanel()
{ // make Mnenhy available in callback functions
  parent.goMnenhy = goMnenhy;
  // init the hPrefWindow if necessary
  if (!parent.hPrefWindow)
    if ("onLoad" in parent)
      parent.onLoad();  // Mozilla 1.8
    else
      parent.Startup(); // Mozilla 1.6
  // init this panel
  parent.gksMnenhyHeadersPanelURL = "chrome://mnenhy-headers/content/mnenhy-headers-pref.xul";
  parent.initPanel(parent.gksMnenhyHeadersPanelURL);
  InitMnenhyPrefHeader(60);
  // register pref exit functions
  parent.hPrefWindow.registerOKCallbackFunc    (SaveMnenhyPrefHeader);
  parent.hPrefWindow.registerCancelCallbackFunc(ExitMnenhyPrefHeader);
}


function InitMnenhyPrefHeader(iDelay)
{
  goMnenhy.SetWaitCursor(window);

  // mnenhy's rdf datasource needs to be ready
  if (!goMnenhy.m_oRDF.IsGood() || !goListbox)
  {
    goMnenhy.Log("InitMnenhyPrefHeader("+iDelay+")");
    goMnenhy.SetWaitCursor(window);
    if (iDelay>0)
      setTimeout(InitMnenhyPrefHeader, 50, iDelay - 1);
    else
      alert(goMnenhy.GetString("RDF_too_slow", gkoProperties));
    goListbox = goListbox || document.getElementById("mnenhyHeaderListChildren");
  }
  else
  { // init global objects
    goMnenhy.Log("InitMnenhyPrefHeader() ok");

    goDropdown     = document.getElementById("mnenhyHeaderDropdown");
    goDropdownList = document.getElementById("mnenhyHeaderDropdownList");

    // get the known headers and prepare check array
    var asKnownHeaders = goMnenhy.m_oRDF.GetNodeList([gksBaseContainer, gksKnown], true);
    var abCheck = {};
    for (var sHeader in asKnownHeaders)
      abCheck[sHeader]=true;

    // get the preference headers
    // -- get the filter headers
    var sPrefHeaderList = goMnenhy.GetCleanStringPref(gksFilterPref);
    // -- get the compose headers
    // (this Mozilla pref is actually broken: mail header names can contain commas...)
    sPrefHeaderList += ":" + goMnenhy.GetCleanStringPref(gksComposePref).replace(/,/g, ":");
    var asPrefHeaderList = sPrefHeaderList.match(/[^:]+/g);

    // now check if they're already known
    for (var iIndex in asPrefHeaderList)
    {
      var sPrefHeader = asPrefHeaderList[iIndex];
      var bValue = sPrefHeader in abCheck;
      if (!bValue)
      { // not known, so add to abCheck...
        abCheck[sPrefHeader]=true;
        // ...and to rdf
        var asItems = {format: gkrsHeaderVars[0]}; // add as text item
        goMnenhy.m_oRDF.AddNode([gksBaseContainer, gksKnown, sPrefHeader], [gksBaseContainer, gksKnown], asItems);  // add to known
      }
    }

    // show all known headers in the listbox
    FillListbox();

    // fill dropdownbox with header list names
    // - standard items
    AddMenuItem(goDropdown, gksDropPrefix+gksFilter,  'SelectMnenhyHeaderList("'+gksFilter+'")',
                goMnenhy.GetString("list_filter", gkoProperties));
    AddMenuItem(goDropdown, gksDropPrefix+gksCompose, 'SelectMnenhyHeaderList("'+gksCompose+'")',
                goMnenhy.GetString("list_compose", gkoProperties));
    // - custom items
    var iLen = goMnenhy.headers.CustomHeaderViewCount()
    if (iLen)
    {
      goDropdown.appendChild(document.createElement("menuseparator"));

      var asHeaderLists = goMnenhy.m_oRDF.GetNodeList(gksBaseContainer);
      iLen = asHeaderLists.length;
      // get all custom lists, but ignore the known headers
      for (var iList = 0; iList < iLen; ++iList)
      {
        var node = asHeaderLists[iList];
        if (node != gksKnown)  // ignore some header lists; currently only one
        {
          var prettyname = goMnenhy.m_oRDF.GetItemValue("name", [gksBaseContainer, node]);
          AddMenuItem(goDropdown, gksDropPrefix+node, "SelectMnenhyHeaderList('" + node + "')",
                      prettyname, null);
        }
      }
      const kiSelected = goMnenhy.headers.GetHeaderView() - goMnenhy.headers.gkiViewCustomBase;
      if (kiSelected >= 0)
      { // have custom list
        goDropdownList.selectedIndex = kiSelected + 3;
        const ksInternalName = gksCustom + kiSelected;
        SelectMnenhyHeaderList(ksInternalName);
      }
    }
    goMnenhy.UnsetWaitCursor(window);
  }
}


// set persistant data in nsPrefWindow
function SetFields(aoPageData)
{
  const koDM = parent.hPrefWindow.wsm.dataManager;
  var oData = koDM.getPageData(parent.gksMnenhyHeadersPanelURL);
  if ("initialized" in oData && oData.initialized)
    return true;   // use stored data

  if ("elementIDs" in oData)
    oData = oData.elementIDs; // bird widget state manager
  else
    oData = aoPageData;       // Seamonkey widget state manager

  // init the page data: transfer the rebuild function for the cancel callback
  oData['SomeNonExistentMnenhyHeaderPrefID'] = {ForceHeaderRebuild: ForceHeaderRebuild};
  return true;
}


// force save when leaving via pref panel OK button
// (this shouldn't actually be needed, but we want make sure ;-) )
function SaveMnenhyPrefHeader()
{
  parent.goMnenhy.m_oRDF.FlushDataSource();
}


// force restoration when leaving via pref panel Cancel button
function ExitMnenhyPrefHeader()
{
  parent.goMnenhy.m_oRDF.RefreshDataSource(true);
  // we have no (easy) possibility to decide if the view is "dirty",
  // so we always force a rebuild
  const koDM = parent.hPrefWindow.wsm.dataManager;
  var oData = koDM.getPageData(parent.gksMnenhyHeadersPanelURL);
  if ("initialized" in oData && oData.initialized)
  { 
    if ("elementIDs" in oData)
      oData = oData.elementIDs; // bird widget state manager
    // ForceHeaderRebuild
    self.goMnenhy = parent.goMnenhy;
    oData['SomeNonExistentMnenhyHeaderPrefID'].ForceHeaderRebuild();
  } // is inited
}


// show all known headers in the listbox
function FillListbox()
{ // hide it
  goListbox.hidden = true;
  // empty the listbox
  EmptyListbox(goListbox);
  // refill it
  var asKnownHeaders = goMnenhy.m_oRDF.GetNodeList([gksBaseContainer, gksKnown]);
  asKnownHeaders.sort();
  var iLen = asKnownHeaders.length;
  for (var iHeader = 0; iHeader < iLen; ++iHeader)
  {
    var sh = asKnownHeaders[iHeader];
    // get the style classes for this header:
    var sClass = "";
    var sPrettyHeader = goMnenhy.Capitalize(sh);
    // -- normal = bold
    if (goMnenhy.m_oRDF.GetItemValue("hl_normal",[gksBaseContainer, gksKnown, sh]))
      sClass += " mnenhy-lh-bold";
    // -- filter = italics
    if (goMnenhy.m_oRDF.GetItemValue("hl_filter",[gksBaseContainer, gksKnown, sh]))
      sClass += " mnenhy-lh-italic";
    // -- compose = small caps
    if (goMnenhy.m_oRDF.GetItemValue("hl_compose",[gksBaseContainer, gksKnown, sh]))
    { // Mozilla won't do small-caps in listboxes/trees, so we've got to fake it
      // we can't underline, since this underlines white space also
      sPrettyHeader = sPrettyHeader.toUpperCase();
    }

    AddListItem(goListbox, gksListPrefix+sh,
                sPrettyHeader, sClass,
                gkrsHeader[goMnenhy.m_oRDF.GetItemValue(gksFormat,[gksBaseContainer, gksKnown, sh])], null,
                goMnenhy.m_oRDF.GetItemValue(gksDisplay, [gksBaseContainer, gksKnown, sh]),
                goMnenhy.m_oRDF.GetItemValue(gksIndex,   [gksBaseContainer, gksKnown, sh]) || sh,
                true, false);
  }
  // show it
  goListbox.hidden = false;
}



// create a listitem in goListbox with id=sId and three cells sLabel, sType and sIndex
// bDisabled can be used to disable the listitem, bChecked governs the checkbox default state
function AddListItem(oListbox, sId,
                     sLabel, sLabelStyleClass,
                     sType,  sTypeStyleClass,
                     sDisplay, sIndex,
                     bDisabled, bChecked)
{
  var oListitem = document.createElement("treeitem");
  oListitem.setAttribute("id", sId);

  var sCheckId   = sId + gksCheckPostfix;
  var sFormatId  = sId + gksFormatPostfix;
  var sDisplayId = sId + gksDisplayPostfix;
  var sIndexId   = sId + gksIndexPostfix;

  var oListrow = document.createElement("treerow");
  oListitem.appendChild(oListrow);

  var oListcell = document.createElement("treecell");
  oListcell.setAttribute("id",        sCheckId);
  oListcell.setAttribute("label",     sLabel);
  oListcell.setAttribute(gksAttrProp, sLabelStyleClass);
  oListrow.appendChild(oListcell);

  oListcell = document.createElement("treecell");
  oListcell.setAttribute("id",        sFormatId);
  oListcell.setAttribute("label",     sType);
  oListcell.setAttribute(gksAttrProp, sTypeStyleClass);
  oListrow.appendChild(oListcell);

  oListcell = document.createElement("treecell");
  oListcell.setAttribute("id",        sDisplayId);
  oListcell.setAttribute("label",     sDisplay);
  oListrow.appendChild(oListcell);

  oListcell = document.createElement("treecell");
  oListcell.setAttribute("id",        sIndexId);
  oListcell.setAttribute("label",     sIndex);
  oListrow.appendChild(oListcell);

  oListbox.appendChild(oListitem);

  SetListcell(sCheckId,   bChecked, bDisabled);
  SetListcell(sFormatId,  null,     bDisabled);
  SetListcell(sDisplayId, null,     bDisabled);
  SetListcell(sIndexId,   null,     bDisabled);
}


// create a menuitem in goDropdown with id=sId and label sLabel
function AddMenuItem(oDropdown, sId, sCommand, sLabel, sLabelStyle)
{
  var oMenuitem = document.createElement("menuitem");
  oMenuitem.setAttribute("id",       sId);
  oMenuitem.setAttribute("label",    sLabel);
  oMenuitem.setAttribute("oncommand",sCommand);
  oDropdown.appendChild(oMenuitem);
  return oMenuitem;
}


// set 'checkbox' and 'disabled' of a treecell
// a value of |null| will not set the respective attribute values
function SetListcell(sListcell, bChecked, bDisabled)
{
  var oCell = document.getElementById(sListcell);
  if (oCell)
  { // 'checkbox' status
    var sProp = String(oCell.getAttribute(gksAttrProp))
                            .replace(gksHeaderActive,    "")
                            .replace(gksHeaderActiveYes, "")
                            .replace(gksAttrDisabled,  "");
    if (bChecked !== null)
      sProp += " " + gksHeaderActive + (bChecked ? (" " + gksHeaderActiveYes) : "");
    if (bDisabled)
      sProp += " " + gksAttrDisabled;
    sProp = sProp.replace(/^\s+|\s+$/g, "").replace(/\s+/g," ");

    oCell.setAttribute(gksAttrProp, sProp);
  }
}


// clear all the checkboxes of the header listbox
function ClearListCheckboxes(oListbox)
{
  var iLen = oListbox.childNodes.length;
  for (var iRow = 0; iRow < iLen; ++iRow)
  {
    var oItem = oListbox.childNodes[iRow];
    SetListcell(oItem.id + gksCheckPostfix, false);
    SetListcell(oItem.id + gksFormatPostfix, null);
  }
}


// set those checkboxes of the header listbox that are specified in the array
// if bDisable is specified, the listitem attribute will be set accordingly
function SetListCheckboxes(sListitemPrefix, asHeaderList, bDisable)
{
  if (asHeaderList)
  {
    var iLen = asHeaderList.length;
    for (var iHeader = 0; iHeader < iLen; ++iHeader)
    {
      var sListitem = sListitemPrefix + asHeaderList[iHeader];
      SetListcell(sListitem + gksCheckPostfix,  true, bDisable);
      SetListcell(sListitem + gksFormatPostfix, null, bDisable);
    }
  }
}


// mark headers used by current list
function SelectMnenhyHeaderList(sNode)
{ // first clear the listbox checkboxes
  ClearListCheckboxes(goListbox);
  // then set new status
  gsCurrentListId = sNode;
  var bDisabled = true;
  switch(sNode)
  {
    case gksFilter:
      { // set headers from pref
        var sPrefHeaderList = goMnenhy.GetCleanStringPref(gksFilterPref);
        var asPrefHeaderList = sPrefHeaderList.match(/[^:]+/g);
        SetListCheckboxes(gksListPrefix, asPrefHeaderList, false);
        // set default headers
        // must be set after pref to overrule double entries
        var asKnownHeaderList = goMnenhy.m_oRDF.GetNodeList([gksBaseContainer, gksKnown], false, "hl_filter", true);
        SetListCheckboxes(gksListPrefix, asKnownHeaderList, true);
      }
      break;

    case gksCompose:
      { // set headers from pref
        var sPrefHeaderList2 = goMnenhy.GetCleanStringPref(gksComposePref);
        var asPrefHeaderList2 = sPrefHeaderList2.match(/[^,]+/g);
        SetListCheckboxes(gksListPrefix, asPrefHeaderList2, false);
        // set default headers
        // must be set after pref to overrule double entries
        var asKnownHeaderList2 = goMnenhy.m_oRDF.GetNodeList([gksBaseContainer, gksKnown], false, "hl_compose", true);
        SetListCheckboxes(gksListPrefix, asKnownHeaderList2, true);
      }
      break;

    default:
      { // set custom headers
        var asHeaderList = goMnenhy.m_oRDF.GetNodeList([gksBaseContainer, sNode]);
        SetListCheckboxes(gksListPrefix, asHeaderList);
        bDisabled = false;
      }
  }
  // set broadcaster
  var oBroadcaster = document.getElementById(gksViewListSel);
  oBroadcaster.setAttribute(gksAttrDisabled, bDisabled);
  CheckListitemSelect(null);
}


function SelectListitemByIndex(iIndex)
{
  const kiMax = goListbox.childNodes.length - 1;
  if (iIndex > kiMax)
    iIndex = kiMax;
  if (iIndex >= 0)
    goListbox.parentNode.contentView.selection.select(iIndex);
  goListbox.focus();
}


function SelectListitem(oListitem)
{
  if (oListitem)
  {
    var iItem = goListbox.parentNode.contentView.getIndexOfItem(oListitem);
    SelectListitemByIndex(iItem);
  }
}


function GetSelectedListitem(oListbox)
{
  const kiRow = oListbox.parentNode.contentView.selection.currentIndex;
  const kiMax = goListbox.childNodes.length - 1;
  if ((kiRow < 0) || (kiRow > kiMax))
    return null;
  var oTreeItem = oListbox.parentNode.contentView.getItemAtIndex(kiRow);
  return oTreeItem;
}


// oListitem is disabled if nonexistent or with disabled property
function IsListitemDisabled(oListitem)
{
  var bDisabled = !oListitem;
  if (!bDisabled)
    bDisabled = (oListitem.firstChild.firstChild.getAttribute(gksAttrProp).search(gksAttrDisabled) >= 0);
  return bDisabled;
}


// listitem click event: check for checkbox
function CheckListitemClick(event)
{
  if (!event.button)
  { // where did the left button hit?
    var oRow = {}, oColID = {}, oChildElt = {};
    goListbox.parentNode.treeBoxObject.getCellAt(event.clientX, event.clientY, oRow, oColID, oChildElt);
    if (oChildElt.value == gksTreeCellImage)
      ToggleHeaderItem();
  } // left button
}


// listitem keydown event: check for VK_SPACE
function CheckListitemKeyDown(event)
{
  if (event.keyCode == 32)  // Space in listbox!
    ToggleHeaderItem();
}


function CheckListitemSelect(event_is_ignored)
{ // remove header and edit header are only allowed if something is selected,
  var oTreeItem = GetSelectedListitem(goListbox);
  var bDisabled = IsListitemDisabled(oTreeItem);
  var oBroadcasterEdit = document.getElementById(gksHeaderListSel);
  oBroadcasterEdit.setAttribute(gksAttrDisabled, !oTreeItem);
}


// if we have listitems and nothing is selected, select the first
function CheckListFocus(event_is_ignored)
{
  if (!GetSelectedListitem(goListbox))
    SelectListitemByIndex(0);
}


// toggle the header checkbox if not disabled
function ToggleHeaderItem()
{
  var oTreeItem = GetSelectedListitem(goListbox);
  if (oTreeItem && !IsListitemDisabled(oTreeItem))
  {
    var oCellNode   = document.getElementById(oTreeItem.id + gksCheckPostfix);
    var sHeader     = oTreeItem.id.substr(gksListPrefix.length);
    var bNotChecked = (oCellNode.getAttribute(gksAttrProp).replace(gksHeaderActive,"").search(gksHeaderActiveYes) < 0);
    SetListcell   (oCellNode.id,             bNotChecked);
    SetHeaderState(gsCurrentListId, sHeader, bNotChecked);
  }
}


// (dis)activate a header in rdf or pref
function SetHeaderState(sCurrentListId, sHeader, bState)
{
  switch (sCurrentListId)
  {
    case gksFilter:
      { // read pref
        var sPref = goMnenhy.GetCleanStringPref(gksFilterPref).replace(/:/g,",");
        // change it
        sPref = SetPrefHeaderState(sPref, bState, sHeader);
        // write pref
        sPref = sPref.replace(/,/g,": ");
        goMnenhy.SetPref(gksFilterPref, goMnenhy.kiPrefString, goMnenhy.Capitalize(sPref));
      }
      break;

    case gksCompose:
      { // read pref
        var sPref2 = goMnenhy.GetCleanStringPref(gksComposePref);
        // change it
        sPref2 = SetPrefHeaderState(sPref2, bState, sHeader);
        // write pref
        goMnenhy.SetPref(gksComposePref, goMnenhy.kiPrefString, goMnenhy.Capitalize(sPref2));
      }
      break;

    default:
      {
        const krsCont   = [gksBaseContainer, sCurrentListId];
        const krsHeader = [gksBaseContainer, sCurrentListId, sHeader];
        if (bState)
          goMnenhy.m_oRDF.AddNode   (krsHeader, krsCont);
        else
          goMnenhy.m_oRDF.RemoveNode(krsHeader);
      }
  }
  // if we're manipulating the current view, always rebuild it
  const kiView = goMnenhy.headers.GetHeaderView() - goMnenhy.headers.gkiViewCustomBase;
  if (kiView >= 0)
  {
    const kiList = goDropdownList.selectedIndex - 3;
    if (kiView == kiList)
      ForceHeaderRebuild(sHeader);
  }
}


// the pref string consists of comma-delimited values
// the changed string will be returned
// bActive controls setting (true) or deleting (false) the sValue
function SetPrefHeaderState(sPref, bActive, sValue)
{
  sPref  = "," + sPref  + ",";
  sValue = "," + sValue + ",";
  if (sPref.indexOf(sValue) != -1)
  { // value found
    if (!bActive)
    { // remove
      sPref = sPref.replace(sValue,",");  // remove item
    }
  }
  else
  { // value not found
    if (bActive)
    { // set
      sPref = sPref.replace(/,/,sValue);  // set as new first item
    }
  }
  return sPref.replace(/^,+|,+$/g, "");   // trim commas
}


// empty the listbox
function EmptyListbox(oListbox)
{
  while (oListbox.childNodes.length)
    oListbox.removeChild(oListbox.lastChild);
}


function AddMnenhyList()
{ // get new list name
  var oName = {value:         "",
               sTitle:        goMnenhy.GetString("addlist_title", gkoProperties),
               sLabelValue:   goMnenhy.GetString("addlist_label", gkoProperties),
               sTextboxValue: ""};
  window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                    "mnenhyHeadersPrefInput",
                    "chrome,modal,centerscreen",
                    oName);
  if (oName.value && oName.value.length)
  { // find first empty custom list
    var iList = 0;
    while (goMnenhy.m_oRDF.ExistNode([gksBaseContainer, gksCustom+iList], [gksBaseContainer]))
      ++iList;
    var sInternalName = gksCustom+iList;
    // add list name to listbox
    if (!goMnenhy.headers.CustomHeaderViewCount())
      goDropdown.appendChild(document.createElement("menuseparator"));
    AddMenuItem(goDropdown, gksDropPrefix+sInternalName, "SelectMnenhyHeaderList('" + sInternalName + "')",
                oName.value, null);
    // add list to rdf
    goMnenhy.m_oRDF.AddContainer([gksBaseContainer, sInternalName], goMnenhy.rdf.gkiRDFContSeq, gksBaseContainer);
    goMnenhy.m_oRDF.ChangeItemValue(oName.value, "name", [gksBaseContainer, sInternalName]);
    // activate the new list
    goDropdownList.selectedIndex = goDropdown.childNodes.length - 1;
    SelectMnenhyHeaderList(sInternalName);
  }
}


function RenameMnenhyList()
{ // get old list name
  var oSelItem = goDropdownList.selectedItem;
  if (oSelItem)
  { // we need an item to rename...
    var sInternalName = oSelItem.id.replace(gksDropPrefix,"");
    // internal name must be customxxx
    if (!sInternalName.indexOf(gksCustom))
    {
      var oName = {value:         "",
                   sTitle:        goMnenhy.GetString("renamelist_title", gkoProperties),
                   sLabelValue:   goMnenhy.GetString("addlist_label",    gkoProperties),
                   sTextboxValue: oSelItem.label};
      window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                        "mnenhyHeadersPrefInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (oName.value && oName.value.length)
      { // rename!
        oSelItem.label = oName.value;
        goMnenhy.m_oRDF.ChangeItemValue(oName.value, "name", [gksBaseContainer, sInternalName]);
        var i = goDropdownList.selectedIndex;
        goDropdownList.selectedIndex = -1;
        goDropdownList.selectedIndex = i;  // refresh dropdownbox
      }
    }
  }
}


function RemoveMnenhyList()
{
  var oSelItem = goDropdownList.selectedItem;
  if (oSelItem)
  { // we need an item to kill...
    var sInternalName = oSelItem.id.replace(gksDropPrefix,"");
    // internal name must be customxxx
    if (!sInternalName.indexOf(gksCustom))
    {
      var oName = {value:"",
                   sTitle:goMnenhy.GetString("removelist_title", gkoProperties),
                   sLabelValue:goMnenhy.GetString("removelist_label", gkoProperties)
                                       .replace("%s",oSelItem.label),
                   bHideTextbox:true};
      window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                        "mnenhyHeadersPrefInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (oName.value)
      { // remove from rdf
        goMnenhy.m_oRDF.RemoveContainer([gksBaseContainer, sInternalName]);
        goMnenhy.m_oRDF.RemoveNode     ([gksBaseContainer, sInternalName]);
        // remove from dropdownbox
        var i = goDropdownList.selectedIndex;
        goDropdownList.removeItemAt(i);
        if (goDropdown.childNodes.length == 3)
        { // no custom headers left
          goDropdownList.removeItemAt(2);  // separator
          i=1;
        }
        // refresh dropdownbox
        if (i < goDropdown.childNodes.length)
          goDropdownList.selectedIndex = i;
        else
          goDropdownList.selectedIndex = i - 1;
        SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(gksDropPrefix,""));
      }
    }
  }
}


function ForceHeaderRebuild(sHeaderName)
{ // force rebuild of all header lists in all mail windows
  var oWM = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
  var oWinList = oWM.getEnumerator(null);
  while (oWinList.hasMoreElements())
  {
    var oWin = oWinList.getNext();
    if ("giHeaderView" in oWin)
    {
      oWin.giHeaderView = goMnenhy.headers.gkiViewDirty; // request header list rebuild
      if (oWin.gDBView)
        oWin.MsgReload(); // reload current message
    }
  }
}


// if the list in the dropdownbox is the current view list, ForceHeaderRebuild
function CheckHeaderRebuild(sHeaderName)
{
  const kiView = goMnenhy.headers.GetHeaderView();
  if (kiView == goMnenhy.headers.gkiViewNoHeaders)
    return; // no rebuild needed

  var bRebuild = (kiView == goMnenhy.headers.gkiViewAllHeaders); // all = always rebuild
  if (bRebuild)
  { // rebuild needed if not inline
    if (goMnenhy.headers.GetHeaderViewInline())
      return; // no rebuild needed
  }
  else
  {
    if (kiView >= goMnenhy.headers.gkiViewCustomBase)
    { // check for header in current custom view
      const ksCont = "header:" + gksCustom + String(kiView - goMnenhy.headers.gkiViewCustomBase);
      bRebuild = !!goMnenhy.m_oRDF.ExistNode(ksCont + ":" + sHeaderName, ksCont);
    }
    else if (    (kiView == goMnenhy.headers.gkiViewNormalMnenhy)
              || (kiView == goMnenhy.headers.gkiViewNormalHeaders)
            )
    { // check for header in normal view
      bRebuild = (goMnenhy.m_oRDF.GetItemValue("hl_normal", "header:known:" + sHeaderName) == "true");
    }
  }
  if (bRebuild)
    ForceHeaderRebuild(sHeaderName);
}


function AddMnenhyHeader()
{ // get new header
  var oName = {value:           "",
               sTitle:          goMnenhy.GetString("add_title", gkoProperties),
               sLabelValue:     goMnenhy.GetString("add_label", gkoProperties),
               sTextboxValue:   "",
               bShowDisplayBox: true,
               sDisplayLabel:   goMnenhy.GetString("edit_display", gkoProperties),
               sDisplayValue:   "",
               bShowIndexBox:   true,
               sIndexLabel:     goMnenhy.GetString("edit_string", gkoProperties),
               sIndexValue:     "",
               bShowListbox:    true,
               sListLabelValue: goMnenhy.GetString("add_listlabel", gkoProperties),
               saList:          gkrsHeader,
               iListSelected:   0};
  window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                    "mnenhyHeadersPrefInput",
                    "chrome,modal,centerscreen",
                    oName);
  if (oName.value && oName.value.name && oName.value.name.length)
  {
    goMnenhy.SetWaitCursor(window);
    var sFormat       = gkrsHeaderVars[oName.value.format];
    var sInternalName = oName.value.name.toLowerCase();
    var sDisplay      = oName.value.display;
    var sIndex        = oName.value.index.toLowerCase();
    if (sIndex == sInternalName)
      sIndex = "";
    // add only if not already there
    if (!goMnenhy.m_oRDF.ExistNode([gksBaseContainer, gksKnown, sInternalName], [gksBaseContainer, gksKnown]))
    { // create new header in rdf
      var asItems = {format:sFormat};
      goMnenhy.m_oRDF.AddNode([gksBaseContainer, gksKnown, sInternalName], [gksBaseContainer, gksKnown], asItems);  // add to known
    }
    // set useToggle mode
    var bUseToggle = (oName.value.format == gkoHeaderPos.mail);
    goMnenhy.m_oRDF.ChangeItemValue(bUseToggle, gksUseToggle, [gksBaseContainer, gksKnown, sInternalName]);
    // set display name
    goMnenhy.m_oRDF.ChangeItemValue(sDisplay,   gksDisplay,   [gksBaseContainer, gksKnown, sInternalName]);
    // set sort string
    goMnenhy.m_oRDF.ChangeItemValue(sIndex,     gksIndex,     [gksBaseContainer, gksKnown, sInternalName]);
    // refill listbox
    FillListbox(goListbox);
    if (goDropdownList.selectedItem)
      SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(gksDropPrefix, ""));
    // select new header, but don't checkmark it
    var oItem = document.getElementById(gksListPrefix + sInternalName);
    SelectListitem(oItem);

    goMnenhy.UnsetWaitCursor(window);
  }
}


function RemoveMnenhyHeader()
{
  var oSelItem = GetSelectedListitem(goListbox);
  if (oSelItem)
  {
    var oChkCell = document.getElementById(oSelItem.id + gksCheckPostfix);
    if (oChkCell)
    {
      var sInternalName = oSelItem.id.replace(gksListPrefix,"");
      var oName = {value:       "",
                   sTitle:      goMnenhy.GetString("remove_title", gkoProperties),
                   sLabelValue: goMnenhy.GetString("remove_label", gkoProperties)
                                        .replace("%s",oChkCell.getAttribute("label")),
                   bHideTextbox:true};
      window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                        "mnenhyHeadersPrefInput",
                        "chrome,modal,centerscreen",
                        oName);
      if (oName.value)
      { // remove all references to this header
        goMnenhy.SetWaitCursor(window);
        SetHeaderState(gksFilter,  sInternalName, false);
        SetHeaderState(gksCompose, sInternalName, false);
        var iMaxCustom = goMnenhy.headers.CustomHeaderViewCount();
        for (var i = 0; i < iMaxCustom; ++i)
          SetHeaderState(gksCustom+i, sInternalName, false);
        // remove from known
        goMnenhy.m_oRDF.RemoveNode([gksBaseContainer, gksKnown, sInternalName]);
        // remember index in listbox
        const kiItem = goListbox.parentNode.contentView.getIndexOfItem(oSelItem);
        // refill listbox
        FillListbox(goListbox);
        if (goDropdownList.selectedItem)
          SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(gksDropPrefix, ""));
        // select another header
        SelectListitemByIndex(kiItem);
        // always update displays
        ForceHeaderRebuild(sInternalName);
        goMnenhy.UnsetWaitCursor(window);
      }
    }
  }
}


function EditMnenhyHeader()
{ // change the view format of the header
  var oSelItem = GetSelectedListitem(goListbox);
  if (oSelItem)
  {
    var sInternalName = oSelItem.id.replace(gksListPrefix,"");
    var iFormat       = gkoHeaderPos[goMnenhy.m_oRDF.GetItemValue(gksFormat, [gksBaseContainer, gksKnown, sInternalName])];
    var sDisplayValue = goMnenhy.m_oRDF.GetItemValue(gksDisplay, [gksBaseContainer, gksKnown, sInternalName]);
    var sIndexValue   = goMnenhy.m_oRDF.GetItemValue(gksIndex,   [gksBaseContainer, gksKnown, sInternalName]) || sInternalName;
    var oName = {value:           "",
                 sTitle:          goMnenhy.GetString("edit_title",  gkoProperties),
                 sLabelValue:     goMnenhy.GetString("edit_label",  gkoProperties),
                 sTextboxValue:   goMnenhy.Capitalize(sInternalName),
                 bShowDisplayBox: true,
                 sDisplayLabel:   goMnenhy.GetString("edit_display", gkoProperties),
                 sDisplayValue:   sDisplayValue,
                 bShowIndexBox:   true,
                 sIndexLabel:     goMnenhy.GetString("edit_string", gkoProperties),
                 sIndexValue:     sIndexValue,
                 bShowListbox:    true,
                 bDisableTextbox: true,
                 sListLabelValue: goMnenhy.GetString("add_listlabel", gkoProperties),
                 saList:          gkrsHeader,
                 iListSelected:   iFormat};
    window.openDialog("chrome://mnenhy-headers/content/mnenhy-headers-pref-input.xul",
                      "mnenhyHeadersPrefInput",
                      "chrome,modal,centerscreen",
                      oName);
    if (oName.value && ((oName.value.format != iFormat) || (oName.value.index != sIndex)))
    {
      goMnenhy.SetWaitCursor(window);
      // set header format
      goMnenhy.m_oRDF.ChangeItemValue(gkrsHeaderVars[oName.value.format], gksFormat, [gksBaseContainer, gksKnown, sInternalName]);
      // set display name
      var sDisplay = oName.value.display;
      goMnenhy.m_oRDF.ChangeItemValue(sDisplay, gksDisplay, [gksBaseContainer, gksKnown, sInternalName]);
      // set sort string
      var sIndex = oName.value.index.toLowerCase();
      if (sIndex == sInternalName)
        sIndex = "";
      goMnenhy.m_oRDF.ChangeItemValue(sIndex, gksIndex , [gksBaseContainer, gksKnown, sInternalName]);
      // set useToggle mode
      var bUseToggle = (oName.value.format == gkoHeaderPos.mail);
      goMnenhy.m_oRDF.ChangeItemValue(bUseToggle, gksUseToggle, [gksBaseContainer, gksKnown, sInternalName]);
      // refill listbox
      FillListbox(goListbox);
      if (goDropdownList.selectedItem)
        SelectMnenhyHeaderList(goDropdownList.selectedItem.id.replace(gksDropPrefix, ""));
      // select the header
      SelectListitem(oSelItem);
      // sometimes update displays
      CheckHeaderRebuild(sInternalName);
      goMnenhy.UnsetWaitCursor(window);
    }
  }
}
