/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gfMnenySidebarDebug = function(ignored){};

function MnenhySidebarInit(aoEventIgnored)
{ // init the debug mode
  const gksPrefDebug = "extensions.mnenhy.log.sidebar";
  if (goMnenhy.GetPref(this.gksPrefDebug, goMnenhy.kiPrefBool))
    gfMnenySidebarDebug = function(asText){goMnenhy.Log(asText);};
  gfMnenySidebarDebug("mnenhMnenhySidebarInit+");
  
  // fix context menu
  nsContextMenu.prototype.showItem = function(aoItemOrId, abShow)
  { // We have this problem with the original function in MailNews:
    // The cookieContextOverlay overlays twice the contentAreaContextMenu and
    // the messagePaneContext, hence its IDs appear even thrice in the same window!
    // So we need to:
    // - search only the child elements for a certain ID
    // - disable all IDs but the first
    // Yeah, I know that this is ugly :(
    const kroItems = (aoItemOrId.constructor == String)
                   ? (this.menu.getElementsByAttribute("id", aoItemOrId) || [])
                   : [aoItemOrId];
    for (var o in kroItems)
    {
      kroItems[o].hidden = !abShow;
      if (abShow)
        abShow = false; // disable the rest
    }
  };
    
  // if we have dynamic 3pane layout, move some boxes around
  const koMailContent       = document.getElementById("mailContent");
  const koMnenhyMailContent = document.getElementById("mnenhyMailContent");
  if (koMailContent)
  { // new dynamic 3pane layout: move mailContent into new hbox
    koMailContent.parentNode.removeChild(koMailContent);
    koMnenhyMailContent.appendChild(koMailContent);
    // overlay the dynamic 3pane change function
    this.MnenhyOldUpdateMailPaneConfig = this.UpdateMailPaneConfig;
    this.UpdateMailPaneConfig = this.MnenhyUpdateMailPaneConfig;
    // move folderTree
    MnenhyPlaceFolderTree();
    // kill "Folder Pane F9" menu item
    document.getElementById("menu_showFolderPane").hidden = true;
  }
  else
  { // old static layout: kill second sidebar
    koMnenhyMailContent.parentNode.removeChild(koMnenhyMailContent);
  }

  // init sidebar ...
  window.removeEventListener("load", sidebar_overlay_init, false);
  sidebar_overlay_init();
  // ... but do our own splitter settings
  const koSplitter = document.getElementById("sidebar-panels-splitter");
  const koSideBox  = document.getElementById('sidebar-panels-splitter-box');
  if (    koSplitter && koSplitter.getAttribute("state") != "collapsed"
       && koSideBox  && koSideBox .getAttribute("collapsed") == "true"
     )
  {  
    koSplitter.hidden = true;
  }

  // initialize the FolderPaneOnClick event listener
  OnLoadFolderPane();
  window.content = GetMessagePane();  
  gfMnenySidebarDebug("MnenhySidebarInit-");
}


function MnenhyUpdateMailPaneConfig()
{
  gfMnenySidebarDebug("MnenhyUpdateMailPaneConfig+");
  MnenhyPlaceFolderTree();
  MnenhyOldUpdateMailPaneConfig();
  gfMnenySidebarDebug("MnenhyUpdateMailPaneConfig-");
}


function MnenhyPlaceFolderTree()
{
  gfMnenySidebarDebug("MnenhyPlaceFolderTree+");
  const kbWideLayout = GetMailPaneConfig() == 1;
  const koFolderTree = GetFolderTree();
  koFolderTree.parentNode.removeChild(koFolderTree);
  
  if (kbWideLayout)
  { // move folderTree into folderPaneBox
    const koFolderPane = document.getElementById("folderPaneBox");
    koFolderPane.insertBefore(koFolderTree, koFolderPane.firstChild);
  }
  else
  { // move folderTree into sidebar-box
    const koSidebar = document.getElementById("sidebar-box");
    koSidebar.insertBefore(koFolderTree, koSidebar.firstChild);
  }
  // hide folderPaneBox and splitter, if necessary
  document.getElementById("folderPaneBox"          ).hidden = !kbWideLayout;
  document.getElementById("sidebar-panels-splitter").hidden =  kbWideLayout;
  const kroSplitters = document.getElementsByAttribute("id", "gray_vertical_splitter");
  if (kroSplitters && kroSplitters.length > 1)
    kroSplitters[1].hidden = !kbWideLayout;
  // reroot messagepane
  window.frames["accountCentralPane"] = document.getElementsByAttribute("name", "accountCentralPane")[0].contentWindow;
  messenger.SetWindow(null, null);
  messenger.SetWindow(window, msgWindow);
  gfMnenySidebarDebug("MnenhyPlaceFolderTree-");
}


// |
// | global init call
// |
window.addEventListener("load", MnenhySidebarInit, false);
